Package: RcppRoll
Type: Package
Title: Efficient Rolling / Windowed Operations
Version: 0.4.0
Authors@R: person("Kevin", "Ushey", role = c("aut", "cre"),
    email = "kevinushey@gmail.com")
Description: Provides fast and efficient routines for
    common rolling / windowed operations. Routines for the
    efficient computation of windowed mean, median,
    sum, product, minimum, maximum, standard deviation
    and variance are provided.
License: GPL (>= 2)
URL: https://kevinushey.github.io/RcppRoll/,
        https://github.com/kevinushey/RcppRoll
BugReports: https://github.com/kevinushey/RcppRoll/issues
Depends: R (>= 2.15.1)
Suggests: zoo, testthat
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-09-04 20:53:14 UTC; kevin
Author: Kevin Ushey [aut, cre]
Maintainer: Kevin Ushey <kevinushey@gmail.com>
Repository: CRAN
Date/Publication: 2026-09-06 07:50:08 UTC

# Generated by roxygen2: do not edit by hand

export(roll_max)
export(roll_maxl)
export(roll_maxr)
export(roll_mean)
export(roll_meanl)
export(roll_meanr)
export(roll_median)
export(roll_medianl)
export(roll_medianr)
export(roll_min)
export(roll_minl)
export(roll_minr)
export(roll_prod)
export(roll_prodl)
export(roll_prodr)
export(roll_sd)
export(roll_sdl)
export(roll_sdr)
export(roll_sum)
export(roll_suml)
export(roll_sumr)
export(roll_var)
export(roll_varl)
export(roll_varr)
importFrom(Rcpp,evalCpp)
useDynLib(RcppRoll, .registration = TRUE)

RcppRoll
========

[![CRAN status](https://www.r-pkg.org/badges/version/RcppRoll)](https://cran.r-project.org/package=RcppRoll)
[![R-CMD-check](https://github.com/kevinushey/RcppRoll/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/kevinushey/RcppRoll/actions/workflows/R-CMD-check.yaml)

This package provides windowed-versions of commonly-used mathematical
and statistical functions.

Install the latest release from CRAN with:

    install.packages("RcppRoll")

Or, install the development version with:

    install_github("kevinushey/RcppRoll")

## Parallelism

When compiled with OpenMP support, RcppRoll parallelizes its rolling
computations across threads. Results do not depend on the number of threads
used. By default, the thread count is chosen by the OpenMP runtime
(typically controllable through the `OMP_NUM_THREADS` environment
variable); pin it with `options(RcppRoll.threads = <n>)`, or set it to 1 to
disable parallelism. Check what your installation supports with:

    RcppRoll::roll_threads()

which reports the number of threads to be used, or `NA` if the package was
compiled without OpenMP support. The package reports this on attach as
well; use `options(RcppRoll.quiet = TRUE)` to suppress the startup message.

The toolchains normally used on Linux and Windows support OpenMP out of the
box, so installations from sources just work. Apple's macOS toolchain ships
without OpenMP support; the R project documents how to enable it at
<https://mac.r-project.org/openmp/>, which provides OpenMP runtimes matching
Apple's compilers and recommends adding

    CPPFLAGS += -Xclang -fopenmp
    LDFLAGS += -lomp

to `~/.R/Makevars`. Alternatively, with Homebrew's OpenMP runtime
(`brew install libomp`), add the following line to `~/.R/Makevars` instead:

    SHLIB_OPENMP_CXXFLAGS = -Xclang -fopenmp -I/opt/homebrew/opt/libomp/include -L/opt/homebrew/opt/libomp/lib -lomp

Either way, reinstall the package from sources afterwards, and confirm the
result with `roll_threads()`.

RcppRoll
========

[![CRAN status](https://www.r-pkg.org/badges/version/RcppRoll)](https://cran.r-project.org/package=RcppRoll)

This package provides windowed-versions of commonly-used mathematical
and statistical functions.

Install the latest release from CRAN with:

    install.packages("RcppRoll")

Or, install the development version with:

    install_github("kevinushey/RcppRoll")

# Auto-generated by internal/make-exports.R

#' RcppRoll
#'
#' Efficient windowed / rolling operations. Each function
#' here applies an operation over a moving window of
#' size \code{n}, with (customizable) weights specified
#' through \code{weights}.
#'
#' The functions postfixed with \code{l} and \code{r}
#' are convenience wrappers that set \strong{l}eft
#' / \strong{r}ight alignment of the windowed operations.
#'
#' @name RcppRoll-exports
#' @param x A numeric vector or a numeric matrix.
#' @param n The window size. Ignored when \code{weights} is non-\code{NULL}.
#' @param weights A vector of length \code{n}, giving the weights for each
#'   element within a window. If \code{NULL}, we take unit weights of width \code{n}.
#' @param by Calculate at every \code{by}-th point rather than every point.
#' @param fill Either an empty vector (no fill), or a vector (recycled to)
#'   length 3 giving left, center and right fills.
#' @param partial Compute windows at the edges of \code{x} over however
#'   many elements are in range, rather than filling them? Cannot be
#'   combined with \code{weights}, and \code{fill} does not apply.
#' @param align Align windows on the \code{"left"}, \code{"center"} or
#'   \code{"right"}.
#' @param normalize Normalize window weights, such that they sum to \code{n}.
#' @param na.rm Remove missing values?
NULL

# Argument checking shared by every wrapper below. 'missing_n' and
# 'missing_fill' come from the caller, since missing() only speaks for the frame
# it is called in; conditions are raised against the caller too, so that a user
# sees the roll_* call they made rather than this helper. Returns the window
# size to use, which 'weights' overrides when the two disagree.
checkRollArgs <- function(n, weights, partial, missing_n, missing_fill) {

  call <- sys.call(-1L)

  if (!is.logical(partial) || length(partial) != 1L || is.na(partial)) {
    stop(simpleError("'partial' should be TRUE or FALSE", call))
  }

  if (partial) {
    if (!is.null(weights))
      stop(simpleError(
        "'partial = TRUE' is not supported together with 'weights'", call))
    if (!missing_fill)
      warning(simpleWarning("'fill' is ignored when 'partial = TRUE'", call))
  }

  if (!missing_n && !is.null(weights) && !isTRUE(length(weights) == n)) {
    warning(simpleWarning(sprintf(
      "'n' is ignored when 'weights' is supplied; using 'n = %i' rather than 'n = %i'",
      length(weights), as.integer(n)
    ), call))
    n <- length(weights)
  }

  # checked after 'weights' has had its say, since empty weights would
  # otherwise smuggle in a window of size zero
  n <- as.integer(n)[1L]
  if (!isTRUE(n >= 1L)) {
    stop(simpleError("'n' should be a positive integer", call))
  }

  n
}
#' @rdname RcppRoll-exports
#' @export
roll_mean <- function(x,
                    n = 1L,
                    weights = NULL,
                    by = 1L,
                    fill = numeric(0),
                    partial = FALSE,
                    align = c("center", "left", "right"),
                    normalize = TRUE,
                    na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_mean_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}

##' @rdname RcppRoll-exports
##' @export
roll_meanr <- function(x,
                     n = 1L,
                     weights = NULL,
                     by = 1L,
                     fill = NA,
                     partial = FALSE,
                     align = "right",
                     normalize = TRUE,
                     na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_mean_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}

##' @rdname RcppRoll-exports
##' @export
roll_meanl <- function(x,
                     n = 1L,
                     weights = NULL,
                     by = 1L,
                     fill = NA,
                     partial = FALSE,
                     align = "left",
                     normalize = TRUE,
                     na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_mean_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}
#' @rdname RcppRoll-exports
#' @export
roll_median <- function(x,
                    n = 1L,
                    weights = NULL,
                    by = 1L,
                    fill = numeric(0),
                    partial = FALSE,
                    align = c("center", "left", "right"),
                    normalize = TRUE,
                    na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_median_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}

##' @rdname RcppRoll-exports
##' @export
roll_medianr <- function(x,
                     n = 1L,
                     weights = NULL,
                     by = 1L,
                     fill = NA,
                     partial = FALSE,
                     align = "right",
                     normalize = TRUE,
                     na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_median_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}

##' @rdname RcppRoll-exports
##' @export
roll_medianl <- function(x,
                     n = 1L,
                     weights = NULL,
                     by = 1L,
                     fill = NA,
                     partial = FALSE,
                     align = "left",
                     normalize = TRUE,
                     na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_median_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}
#' @rdname RcppRoll-exports
#' @export
roll_min <- function(x,
                    n = 1L,
                    weights = NULL,
                    by = 1L,
                    fill = numeric(0),
                    partial = FALSE,
                    align = c("center", "left", "right"),
                    normalize = TRUE,
                    na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_min_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}

##' @rdname RcppRoll-exports
##' @export
roll_minr <- function(x,
                     n = 1L,
                     weights = NULL,
                     by = 1L,
                     fill = NA,
                     partial = FALSE,
                     align = "right",
                     normalize = TRUE,
                     na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_min_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}

##' @rdname RcppRoll-exports
##' @export
roll_minl <- function(x,
                     n = 1L,
                     weights = NULL,
                     by = 1L,
                     fill = NA,
                     partial = FALSE,
                     align = "left",
                     normalize = TRUE,
                     na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_min_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}
#' @rdname RcppRoll-exports
#' @export
roll_max <- function(x,
                    n = 1L,
                    weights = NULL,
                    by = 1L,
                    fill = numeric(0),
                    partial = FALSE,
                    align = c("center", "left", "right"),
                    normalize = TRUE,
                    na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_max_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}

##' @rdname RcppRoll-exports
##' @export
roll_maxr <- function(x,
                     n = 1L,
                     weights = NULL,
                     by = 1L,
                     fill = NA,
                     partial = FALSE,
                     align = "right",
                     normalize = TRUE,
                     na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_max_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}

##' @rdname RcppRoll-exports
##' @export
roll_maxl <- function(x,
                     n = 1L,
                     weights = NULL,
                     by = 1L,
                     fill = NA,
                     partial = FALSE,
                     align = "left",
                     normalize = TRUE,
                     na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_max_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}
#' @rdname RcppRoll-exports
#' @export
roll_prod <- function(x,
                    n = 1L,
                    weights = NULL,
                    by = 1L,
                    fill = numeric(0),
                    partial = FALSE,
                    align = c("center", "left", "right"),
                    normalize = TRUE,
                    na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_prod_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}

##' @rdname RcppRoll-exports
##' @export
roll_prodr <- function(x,
                     n = 1L,
                     weights = NULL,
                     by = 1L,
                     fill = NA,
                     partial = FALSE,
                     align = "right",
                     normalize = TRUE,
                     na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_prod_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}

##' @rdname RcppRoll-exports
##' @export
roll_prodl <- function(x,
                     n = 1L,
                     weights = NULL,
                     by = 1L,
                     fill = NA,
                     partial = FALSE,
                     align = "left",
                     normalize = TRUE,
                     na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_prod_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}
#' @rdname RcppRoll-exports
#' @export
roll_sum <- function(x,
                    n = 1L,
                    weights = NULL,
                    by = 1L,
                    fill = numeric(0),
                    partial = FALSE,
                    align = c("center", "left", "right"),
                    normalize = TRUE,
                    na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_sum_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}

##' @rdname RcppRoll-exports
##' @export
roll_sumr <- function(x,
                     n = 1L,
                     weights = NULL,
                     by = 1L,
                     fill = NA,
                     partial = FALSE,
                     align = "right",
                     normalize = TRUE,
                     na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_sum_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}

##' @rdname RcppRoll-exports
##' @export
roll_suml <- function(x,
                     n = 1L,
                     weights = NULL,
                     by = 1L,
                     fill = NA,
                     partial = FALSE,
                     align = "left",
                     normalize = TRUE,
                     na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_sum_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}
#' @rdname RcppRoll-exports
#' @export
roll_sd <- function(x,
                    n = 1L,
                    weights = NULL,
                    by = 1L,
                    fill = numeric(0),
                    partial = FALSE,
                    align = c("center", "left", "right"),
                    normalize = TRUE,
                    na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_sd_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}

##' @rdname RcppRoll-exports
##' @export
roll_sdr <- function(x,
                     n = 1L,
                     weights = NULL,
                     by = 1L,
                     fill = NA,
                     partial = FALSE,
                     align = "right",
                     normalize = TRUE,
                     na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_sd_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}

##' @rdname RcppRoll-exports
##' @export
roll_sdl <- function(x,
                     n = 1L,
                     weights = NULL,
                     by = 1L,
                     fill = NA,
                     partial = FALSE,
                     align = "left",
                     normalize = TRUE,
                     na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_sd_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}
#' @rdname RcppRoll-exports
#' @export
roll_var <- function(x,
                    n = 1L,
                    weights = NULL,
                    by = 1L,
                    fill = numeric(0),
                    partial = FALSE,
                    align = c("center", "left", "right"),
                    normalize = TRUE,
                    na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_var_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}

##' @rdname RcppRoll-exports
##' @export
roll_varr <- function(x,
                     n = 1L,
                     weights = NULL,
                     by = 1L,
                     fill = NA,
                     partial = FALSE,
                     align = "right",
                     normalize = TRUE,
                     na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_var_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}

##' @rdname RcppRoll-exports
##' @export
roll_varl <- function(x,
                     n = 1L,
                     weights = NULL,
                     by = 1L,
                     fill = NA,
                     partial = FALSE,
                     align = "left",
                     normalize = TRUE,
                     na.rm = FALSE)
{
  n <- checkRollArgs(n, weights, partial, missing(n), missing(fill))

  .Call(
    C_roll_var_impl,
    x,
    as.integer(n),
    as.numeric(weights),
    as.integer(by),
    as.numeric(fill),
    as.logical(partial),
    as.character(match.arg(align)),
    as.logical(normalize),
    as.logical(na.rm)
  )
}

context("threads")

# The drivers walk their windows in chunks, restarting the accumulator at
# every chunk boundary, whether or not those chunks then run on threads.
# These inputs are sized well past the chunk size so that a serial run
# crosses boundaries too.

window_reference <- function(x, idx, n, f) {
  vapply(idx, function(i) f(x[i:(i + n - 1L)]), numeric(1))
}

test_that("chunked window walks agree with per-window computation", {

  set.seed(1)
  x <- rnorm(5e4)
  x[sample(length(x), 500)] <- NA

  n <- 100L
  idx <- sort(sample(length(x) - n + 1L, 200L))

  # without fill, the value at position i is the window starting at i,
  # whatever the alignment
  expect_equal(roll_sum(x, n, na.rm = TRUE)[idx],
               window_reference(x, idx, n, function(w) sum(w, na.rm = TRUE)))
  expect_equal(roll_mean(x, n, na.rm = TRUE)[idx],
               window_reference(x, idx, n, function(w) mean(w, na.rm = TRUE)))
  expect_equal(roll_min(x, n, na.rm = TRUE)[idx],
               window_reference(x, idx, n, function(w) min(w, na.rm = TRUE)))
  expect_equal(roll_max(x, n, na.rm = TRUE)[idx],
               window_reference(x, idx, n, function(w) max(w, na.rm = TRUE)))
  expect_equal(roll_median(x, n, na.rm = TRUE)[idx],
               window_reference(x, idx, n, function(w) median(w, na.rm = TRUE)))
  expect_equal(roll_var(x, n, na.rm = TRUE)[idx],
               window_reference(x, idx, n, function(w) var(w, na.rm = TRUE)))
  expect_equal(roll_sd(x, n, na.rm = TRUE)[idx],
               window_reference(x, idx, n, function(w) sd(w, na.rm = TRUE)))

  # partial windows clip at the edges; check those edges along with the rest
  eidx <- c(1L, 2L, idx, length(x) - 1L, length(x))
  left <- (n - 1L) %/% 2L
  right <- n %/% 2L
  expect_equal(
    roll_sum(x, n, partial = TRUE, na.rm = TRUE)[eidx],
    vapply(eidx, function(i) {
      w <- x[max(1L, i - left):min(length(x), i + right)]
      sum(w, na.rm = TRUE)
    }, numeric(1)))

  # the weighted path walks the same chunks
  y <- rnorm(5e4)
  w <- runif(n)
  scaled <- w / sum(w) * n
  expect_equal(roll_sum(y, weights = w)[idx],
               window_reference(y, idx, n, function(v) sum(v * scaled)))

})

test_that("roll_threads() reports the thread count", {

  old <- options(RcppRoll.threads = NULL)
  on.exit(options(old), add = TRUE)

  threads <- roll_threads()
  expect_true(is.integer(threads) && length(threads) == 1L)
  expect_true(is.na(threads) || threads >= 1L)

  # the option pins the count where OpenMP is available
  if (!is.na(threads)) {
    options(RcppRoll.threads = 3)
    expect_identical(roll_threads(), 3L)
  }

})

test_that("results do not depend on the number of threads", {

  old <- options(RcppRoll.threads = NULL)
  on.exit(options(old), add = TRUE)

  set.seed(42)
  x <- rnorm(5e4)
  x[sample(length(x), 500)] <- NA
  m <- matrix(rnorm(1e5), ncol = 2)
  weights <- runif(100)

  # columns too short to chunk run across threads as whole columns instead
  wm <- matrix(rnorm(1e5), nrow = 100)
  wweights <- runif(20)

  functions <- c("mean", "median", "min", "max", "prod", "sum", "sd", "var")

  for (f in functions) {
    roll <- get(paste("roll", f, sep = "_"), envir = asNamespace("RcppRoll"))

    results <- list()
    for (threads in c(1, 2)) {
      options(RcppRoll.threads = threads)
      results[[threads]] <- list(
        roll(x, 100),
        roll(x, 100, na.rm = TRUE),
        roll(x, 100, fill = NA),
        roll(x, 100, fill = NA, by = 7),
        roll(x, 100, partial = TRUE),
        roll(x, 100, weights = weights, na.rm = TRUE),
        roll(m, 100),
        roll(wm, 20),
        roll(wm, 20, partial = TRUE),
        roll(wm, 20, weights = wweights)
      )
    }

    expect_identical(results[[1]], results[[2]])
  }

})

context("var and sd")

# frequency weights: m = sum(w * x) / sum(w), s2 = sum(w * (x - m)^2) / (sum(w) - 1)
reference_var <- function(window, weights) {
  keep <- !is.na(window)
  window <- window[keep]
  weights <- weights[keep]
  if (length(window) < 2 || !(sum(weights) > 1)) return(NA_real_)
  m <- sum(weights * window) / sum(weights)
  sum(weights * (window - m)^2) / (sum(weights) - 1)
}

# 'normalize = TRUE' scales the weights to sum to n before they reach C++
normalize <- function(weights, n) weights / sum(weights) * n

test_that("weighted roll_var uses a weighted variance, not var(x * weights) (#47)", {

  x <- c(1, 2, 3, 4, 5, 6)
  w <- c(1, 3, 1)

  expect_equal(
    roll_var(x, 3, weights = w),
    vapply(1:4, function(i) reference_var(x[i:(i + 2)], normalize(w, 3)), numeric(1))
  )

  # the old behaviour was var(window * weights), which is a different number
  expect_false(isTRUE(all.equal(
    roll_var(x, 3, weights = w)[1],
    var(x[1:3] * normalize(w, 3))
  )))

})

test_that("equal weights reduce to the unweighted variance (#47)", {

  if (!requireNamespace("zoo", quietly = TRUE))
    skip("zoo not installed")

  set.seed(99)
  x <- rnorm(30)

  for (n in c(2L, 3L, 5L, 8L)) {
    expect_equal(roll_var(x, n, weights = rep(1, n)), roll_var(x, n))
    expect_equal(roll_sd(x, n, weights = rep(1, n)), roll_sd(x, n))
    expect_equal(roll_var(x, n), as.numeric(zoo::rollapply(x, n, var)))
    expect_equal(roll_sd(x, n), as.numeric(zoo::rollapply(x, n, sd)))
  }

})

test_that("weighted roll_var keeps each weight with its own value under na.rm (#47)", {

  set.seed(101)

  for (trial in 1:50) {
    n <- sample(2:6, 1)
    x <- round(rnorm(n + sample(0:8, 1)), 3)
    x[sample(seq_along(x), sample(0:length(x), 1))] <- NA
    w <- round(runif(n, 0.2, 4), 3)

    expect_equal(
      as.numeric(roll_var(x, n, weights = w, na.rm = TRUE)),
      vapply(seq_len(length(x) - n + 1),
             function(i) reference_var(x[i:(i + n - 1)], normalize(w, n)),
             numeric(1))
    )
  }

  # a window that drops its first value must not shift the weights along with it
  y <- c(NA, 10, 20, 30)
  w <- normalize(c(1, 1, 5), 3)
  expect_equal(roll_var(y, 3, weights = c(1, 1, 5), na.rm = TRUE)[1],
               reference_var(c(10, 20), w[2:3]))

})

test_that("roll_var and roll_sd give NA for windows with fewer than two values (#47)", {

  # var() is NA for a vector of length 0 or 1; these used to be 0 or NaN
  expect_equal(roll_var(rep(NA_real_, 4), 3, na.rm = TRUE), rep(NA_real_, 2))
  expect_equal(roll_sd(rep(NA_real_, 4), 3, na.rm = TRUE), rep(NA_real_, 2))
  expect_equal(roll_var(c(NA, NA, 5, NA), 3, na.rm = TRUE), rep(NA_real_, 2))
  expect_equal(roll_var(1:5, 1), rep(NA_real_, 5))

  expect_equal(roll_var(rep(NA_real_, 4), 3, weights = c(1, 1, 1), na.rm = TRUE),
               rep(NA_real_, 2))

  # NA, not NaN
  expect_false(any(is.nan(roll_var(rep(NA_real_, 4), 3, na.rm = TRUE))))

})

test_that("unweighted roll_var matches zoo under both na.rm settings", {

  if (!requireNamespace("zoo", quietly = TRUE))
    skip("zoo not installed")

  set.seed(202)

  for (trial in 1:50) {
    len <- sample(6:30, 1)
    n <- sample(2:5, 1)
    x <- rnorm(len)
    x[sample(len, sample(0:(len %/% 2), 1))] <- NA

    expect_equal(
      roll_var(x, n, na.rm = TRUE),
      as.numeric(zoo::rollapply(x, n, function(w) {
        w <- w[!is.na(w)]
        if (length(w) < 2) NA_real_ else var(w)
      }))
    )

    expect_equal(
      roll_var(x, n, na.rm = FALSE),
      as.numeric(zoo::rollapply(x, n, function(w) if (anyNA(w)) NA_real_ else var(w)))
    )
  }

})

context("partial")

test_that("partial windows compute the edges instead of filling them (#18)", {

  # the sequence and expected output from the issue -- note the reporter says
  # "right align", but their numbers are left-aligned in this package's terms
  x <- c(0, 1, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0)
  expect_equal(roll_suml(x, 3, partial = TRUE),
               c(1, 2, 2, 3, 3, 2, 2, 1, 2, 1, 1, 0))

  # unchanged without partial
  expect_equal(roll_suml(x, 3), c(1, 2, 2, 3, 3, 2, 2, 1, 2, 1, NA, NA))

})

test_that("partial windows match zoo::rollapply(partial = TRUE)", {

  if (!requireNamespace("zoo", quietly = TRUE))
    skip("zoo not installed")

  set.seed(31)
  x <- round(rnorm(14), 3)
  functions <- c("mean", "median", "min", "max", "prod", "sum", "sd", "var")

  for (f in functions) {
    RcppRoll <- get(paste("roll", f, sep = "_"), envir = asNamespace("RcppRoll"))
    for (align in c("left", "center", "right")) {
      for (n in 1:7) {
        expect_equal(
          as.numeric(RcppRoll(x, n, align = align, partial = TRUE)),
          as.numeric(zoo::rollapply(x, n, get(f), align = align, partial = TRUE))
        )
      }
    }
  }

  # a window wider than the data is still computable at every point
  y <- 1:5
  expect_equal(roll_sumr(y, 12, partial = TRUE),
               as.numeric(zoo::rollapply(y, 12, sum, align = "right", partial = TRUE)))
  expect_equal(roll_sumr(y, 12, partial = TRUE), cumsum(y))

})

test_that("an outsized 'n' computes without reserving what it cannot use", {

  # windows are clipped to the data, so a huge nominal 'n' must not size any
  # buffer -- the median accumulator once reserved 'n' doubles for this
  x <- as.numeric(1:10)
  expect_equal(roll_median(x, n = 2e8, partial = TRUE), rep(5.5, 10))
  expect_equal(roll_sum(x, n = 2e8, partial = TRUE), rep(55, 10))

})

test_that("partial output has one element per input element", {

  x <- rnorm(14)
  for (align in c("left", "center", "right"))
    for (n in c(1L, 2L, 5L, 20L))
      expect_equal(length(roll_mean(x, n, align = align, partial = TRUE)), length(x))

})

test_that("var and sd give NA on a one-element partial window", {

  # rather than the NaN that a zero-length denominator used to produce
  expect_true(is.na(roll_sdr(1:6, 3, partial = TRUE)[1]))
  expect_false(is.nan(roll_sdr(1:6, 3, partial = TRUE)[1]))
  expect_true(is.na(roll_varr(1:6, 3, partial = TRUE)[1]))

})

test_that("na.rm still applies inside a truncated window", {

  x <- c(1, NA, 3, 4, 5)
  expect_equal(roll_sumr(x, 3, partial = TRUE, na.rm = FALSE),
               c(1, NA, NA, NA, 12))
  expect_equal(roll_sumr(x, 3, partial = TRUE, na.rm = TRUE),
               c(1, 1, 4, 7, 12))

})

test_that("partial keeps every row for matrices", {

  m <- matrix(1:12, nrow = 6, dimnames = list(NULL, c("A", "B")))
  result <- roll_sum(m, 3, partial = TRUE)

  expect_equal(dim(result), dim(m))
  expect_equal(colnames(result), colnames(m))
  expect_equal(as.numeric(result[, 1]), roll_sum(m[, 1], 3, partial = TRUE))
  expect_equal(as.numeric(result[, 2]), roll_sum(m[, 2], 3, partial = TRUE))

})

test_that("partial computes at strided points when 'by' is given", {

  # points that are skipped are not computed, and 'fill' does not apply
  expect_equal(roll_sumr(1:8, 3, by = 2, partial = TRUE),
               c(1, NA, 6, NA, 12, NA, 18, NA))

})

test_that("partial rejects what it does not support (#18)", {

  expect_error(roll_sum(1:9, 3, weights = c(1, 1, 1), partial = TRUE),
               "not supported together with 'weights'")

  # only TRUE/FALSE -- zoo's numeric 'minimum observations' form is not supported
  expect_error(roll_sum(1:9, 3, partial = 2), "should be TRUE or FALSE")
  expect_error(roll_sum(1:9, 3, partial = NA), "should be TRUE or FALSE")
  expect_error(roll_sum(1:9, 3, partial = c(TRUE, TRUE)), "should be TRUE or FALSE")

  expect_warning(roll_sum(1:9, 3, partial = TRUE, fill = 0),
                 "'fill' is ignored")

  # the r/l wrappers default 'fill' themselves, so they must stay quiet
  expect_silent(roll_sumr(1:9, 3, partial = TRUE))
  expect_silent(roll_suml(1:9, 3, partial = TRUE))

})

test_that("partial = FALSE leaves existing behaviour alone", {

  x <- rnorm(12)
  for (align in c("left", "center", "right")) {
    expect_equal(roll_mean(x, 4, align = align, partial = FALSE),
                 roll_mean(x, 4, align = align))
  }
  expect_equal(roll_sumr(1:9, 3, partial = FALSE), roll_sumr(1:9, 3))
  expect_silent(roll_sum(1:9, 3, partial = FALSE))

})

context("zoo")

test_that("we behave similarly to zoo::rollapply", {

  if (!requireNamespace("zoo", quietly = TRUE))
    skip("zoo not installed")

  library(testthat)

  functions <- c("mean", "median", "prod", "min", "max", "sum")
  x <- rnorm(50)
  window <- 5L

  run_tests <- function(data, width, ..., functions, gctorture = FALSE) {
    for (f in functions) {
      RcppRoll <- get(paste("roll", f, sep = "_"), envir = asNamespace("RcppRoll"))
      zoo <- zoo::rollapply(data, width, FUN = get(f), ...)
      if (is.matrix(zoo)) {
        dimnames(zoo) <- NULL
      }
      if (gctorture) gctorture(TRUE)
      RcppRollRes <- RcppRoll(data, width, ...)
      if (gctorture) gctorture(FALSE)
      withCallingHandlers(
        expect_equal(RcppRollRes, zoo),
        error = function(cnd) {
          str(list(fn = f, data = data, width = width, ...))
        }
      )
    }
  }

  run_tests(x, window, functions = functions)

  window <- 50L
  run_tests(x, window, functions = functions)

  window <- 1L
  run_tests(x, window, functions = functions)

  ## test against small numbers
  x <- rnorm(1E3) ^ 100
  run_tests(x, 5L, functions = functions)

  ## and large numbers
  x <- rnorm(1E3, mean = 1E200, sd = 1E201)
  run_tests(x, 5L, functions = functions)

  ## now let's really stress it...
  args <- expand.grid(
    width = list(3L, 10L, 100L),
    fill = list(NA, c(-1, 0, 1)),
    align = list("left", "center", "right"),
    by = c(1L, 2L, 5L),
    na.rm = c(TRUE, FALSE)
  )

  # don't use median here
  data <- rnorm(1E2, 100, 50)
  for (i in 1:nrow(args)) {
    run_tests(data,
              args$width[[i]],
              fill = args$fill[[i]],
              align = args$align[[i]],
              na.rm = args$na.rm[[i]],
              by = args$by[[i]],
              functions = functions)
  }

  data[sample(length(data), length(data) / 3)] <- NA
  for (i in 1:nrow(args)) {
    suppressWarnings(run_tests(data,
                               args$width[[i]],
                               fill = args$fill[[i]],
                               align = args$align[[i]],
                               na.rm = args$na.rm[[i]],
                               by = args$by[[i]],
                               functions = functions))
  }

  data <- matrix(rnorm(2E2, 100, 50), nrow = 100)
  for (i in 1:nrow(args)) {
    run_tests(
      data, args$width[[i]],
      fill = args$fill[[i]],
      align = args$align[[i]],
      by = args$by[[i]],
      functions = functions
    )
  }

})

test_that("we don't segfault when window size > vector size on ops with fill", {

  x <- c(1:5)
  w <- 10
  gctorture(TRUE)
  result <- roll_meanr(x, w)
  gctorture(FALSE)
  expect_identical(
    roll_meanr(x, w),
    rep(NA_real_, length(x))
  )

})

test_that("we handle an empty fill properly", {

  if (!requireNamespace("zoo", quietly = TRUE))
    skip("zoo not installed")

  for (i in 10:100) {
    data <- 1:i
    lhs <- zoo::rollapply(data, 3, mean, by = 3)
    rhs <- roll_mean(data, 3, by = 3, fill = numeric())
    expect_identical(lhs, rhs)
  }
})

test_that("median handles NAs appropriately", {

  y <- c(NA, 1:3, NA)
  expect_equal(
    roll_median(y, n = 3L, na.rm = TRUE),
    c(1.5, 2.0, 2.5)
  )

})

context("matrix")

test_that("'by' shortens a matrix result the way it shortens a vector one", {

  # the output used to be sized as though 'by' were 1, so a strided matrix
  # result carried rows that were never computed
  m <- matrix(as.numeric(1:24), nrow = 12)

  for (by in c(1L, 2L, 5L)) {
    result <- roll_sum(m, 3, by = by)
    expect_equal(nrow(result), length(roll_sum(m[, 1], 3, by = by)))
    expect_equal(as.numeric(result[, 1]), roll_sum(m[, 1], 3, by = by))
    expect_equal(as.numeric(result[, 2]), roll_sum(m[, 2], 3, by = by))
  }

  # 'fill' and 'partial' still report one row per input row
  expect_equal(nrow(roll_sum(m, 3, by = 2, fill = NA)), nrow(m))
  expect_equal(nrow(roll_sum(m, 3, by = 2, partial = TRUE)), nrow(m))

})

test_that("'weights' sets the window size for matrices too", {

  # 'n' is left at its default here, so the result used to be sized for n = 1
  # and its last row read off the end of each column
  m <- matrix(as.numeric(1:12), nrow = 6)

  result <- roll_mean(m, weights = c(1, 1))
  expect_equal(dim(result), c(5L, 2L))
  expect_equal(as.numeric(result[, 1]), roll_mean(m[, 1], weights = c(1, 1)))
  expect_equal(as.numeric(result[, 2]), roll_mean(m[, 2], weights = c(1, 1)))

  for (f in c("mean", "median", "min", "max", "prod", "sum", "sd", "var")) {
    roll <- get(paste("roll", f, sep = "_"), envir = asNamespace("RcppRoll"))
    result <- roll(m, weights = c(1, 3, 1))
    expect_equal(dim(result), c(4L, 2L))
    expect_equal(as.numeric(result[, 1]), roll(m[, 1], weights = c(1, 3, 1)))
  }

})

test_that("matrix columns are rolled independently of one another", {

  set.seed(41)
  m <- matrix(rnorm(60), nrow = 20, dimnames = list(NULL, c("a", "b", "c")))
  m[c(3, 25, 48)] <- NA

  for (f in c("mean", "median", "min", "max", "sum", "sd", "var")) {
    roll <- get(paste("roll", f, sep = "_"), envir = asNamespace("RcppRoll"))
    for (na.rm in c(TRUE, FALSE)) {
      result <- roll(m, 4, na.rm = na.rm)
      expect_equal(colnames(result), colnames(m))
      for (j in 1:3)
        expect_equal(as.numeric(result[, j]), roll(m[, j], 4, na.rm = na.rm))
    }
  }

})

context("degenerate sizes")

test_that("a window wider than the data gives an empty result without fill", {

  # integer division used to round the window count back up to one when 'by'
  # exceeded the shortfall, and that window read past the data
  expect_equal(roll_sum(as.numeric(1:5), n = 7, by = 3), numeric(0))
  expect_equal(roll_sum(as.numeric(1:5), n = 7), numeric(0))

  m <- matrix(as.numeric(1:4), nrow = 2)
  expect_equal(dim(roll_sum(m, n = 5, by = 10)), c(0L, 2L))

  # with 'fill' there is one output per input, all filled
  expect_equal(roll_sum(as.numeric(1:5), n = 7, fill = NA), rep(NA_real_, 5))

})

test_that("a window of size zero or less is rejected", {

  expect_error(roll_sum(1:20, n = 0, fill = NA), "positive integer")
  expect_error(roll_sum(1:20, n = -1), "positive integer")
  expect_error(roll_sum(1:20, n = NA_integer_), "positive integer")

  # empty weights used to smuggle in a window of size zero
  expect_error(
    suppressWarnings(roll_sum(1:20, n = 5, weights = numeric(0))),
    "positive integer")

})

test_that("higher-dimensional arrays are rejected rather than flattened", {

  a <- array(as.numeric(1:8), c(2, 2, 2), dimnames = list(NULL, c("a", "b"), NULL))
  expect_error(roll_mean(a, 2), "numeric vector or matrix")

  # a one-dimensional array still rolls as a plain vector
  expect_equal(roll_sum(array(as.numeric(1:5), 5), 2), c(3, 5, 7, 9))

})

context("conditions")

# The wrappers share one argument-checking helper, so its errors and warnings
# have to be reported against the roll_* call the user actually made rather
# than against the helper.

test_that("errors name the roll_* function they came from", {

  condition <- tryCatch(roll_sum(1:9, 3, partial = 2), error = function(e) e)
  expect_s3_class(condition, "error")
  expect_identical(as.character(conditionCall(condition)[[1L]]), "roll_sum")

  condition <- tryCatch(
    roll_medianr(1:9, 3, weights = c(1, 1, 1), partial = TRUE),
    error = function(e) e)
  expect_identical(as.character(conditionCall(condition)[[1L]]), "roll_medianr")

})

test_that("warnings name the roll_* function they came from", {

  condition <- tryCatch(roll_sum(1:9, n = 2, weights = 1), warning = function(w) w)
  expect_s3_class(condition, "warning")
  expect_identical(as.character(conditionCall(condition)[[1L]]), "roll_sum")

  condition <- tryCatch(
    roll_varl(1:9, 3, partial = TRUE, fill = 0), warning = function(w) w)
  expect_identical(as.character(conditionCall(condition)[[1L]]), "roll_varl")

})

test_that("the checking helper is not exported", {

  expect_false("checkRollArgs" %in% getNamespaceExports("RcppRoll"))

})

context("Weights")

test_that("roll_* do not mutate weights vector", {
  d <- data.frame(w = c(0.2, 0.1, 0.1, 0.05, 0.05))
  roll_sum(1:25, n = length(d$w), weights = d$w)
  expect_identical(d$w, c(0.2, 0.1, 0.1, 0.05, 0.05))
})

test_that("weighted roll_mean matches weighted.mean over zoo's windows (#23)", {

  if (!requireNamespace("zoo", quietly = TRUE))
    skip("zoo not installed")

  # NOTE: zoo::rollapply() builds variable-width windows when handed a vector as
  # its 'width', so it is not a direct analogue for our 'weights'. Instead, we
  # let zoo generate the windows and use weighted.mean() as the reference.
  reference <- function(x, n, weights, align, na.rm) {
    fill <- if (align == "center") NULL else NA
    zoo::rollapply(
      x, n,
      function(window) weighted.mean(window, weights, na.rm = na.rm),
      align = align, fill = fill
    )
  }

  set.seed(123)
  x <- rnorm(50)
  x[c(4, 5, 6, 7, 20, 21, 33)] <- NA

  weights <- list(c(1, 3, 3, 1), c(2, 1, 4), c(1, 2, 3, 2, 1), 5)
  for (w in weights) {
    n <- length(w)
    for (align in c("left", "center", "right")) {
      for (na.rm in c(TRUE, FALSE)) {
        expect_equal(
          roll_mean(x, n, weights = w, align = align, na.rm = na.rm,
                    fill = if (align == "center") numeric() else NA),
          as.numeric(reference(x, n, w, align, na.rm))
        )
      }
    }
  }

})

test_that("weighted roll_mean re-normalizes weights after dropping NAs (#23)", {
  tmp <- c(1, 1, 1, 1, NA, NA, NA, NA, 1, 1)
  expect_equal(
    roll_mean(tmp, 4, c(1, 3, 3, 1), na.rm = TRUE),
    c(1, 1, 1, 1, NaN, 1, 1)
  )
  # an all-NA window gives NaN, as in the unweighted case
  expect_equal(
    roll_mean(tmp, 4, c(1, 3, 3, 1), na.rm = TRUE),
    roll_mean(tmp, 4, na.rm = TRUE)
  )
})

test_that("weighted roll_median with equal weights matches zoo::rollapply", {

  if (!requireNamespace("zoo", quietly = TRUE))
    skip("zoo not installed")

  # NOTE: only for odd-sized windows of non-NA data -- a weighted median selects
  # an observation rather than interpolating between the two middle values.
  set.seed(456)
  x <- rnorm(50)

  for (n in c(1L, 3L, 5L, 7L)) {
    expect_equal(
      roll_medianr(x, n, weights = rep(1, n)),
      as.numeric(zoo::rollapply(x, n, median, align = "right", fill = NA))
    )
    expect_equal(
      roll_medianr(x, n, weights = rep(1, n)),
      roll_medianr(x, n)
    )
  }

})

test_that("weighted roll_median pairs weights with their own values", {

  if (!requireNamespace("zoo", quietly = TRUE))
    skip("zoo not installed")

  reference <- function(window, weights) {
    keep <- !is.na(window)
    window <- window[keep]
    weights <- weights[keep]
    if (!length(window)) return(NA_real_)
    ordered <- order(window)
    window <- window[ordered]
    weights <- weights[ordered]
    window[which(cumsum(weights) >= sum(weights) / 2)[1]]
  }

  set.seed(789)
  x <- rnorm(50)
  x[c(3, 11, 12, 13, 14, 30)] <- NA

  for (w in list(c(1, 2, 3, 2, 1), c(10, 1, 1), c(1, 1, 10))) {
    n <- length(w)
    for (align in c("left", "center", "right")) {
      expect_equal(
        roll_median(x, n, weights = w, align = align, na.rm = TRUE,
                    fill = if (align == "center") numeric() else NA),
        as.numeric(zoo::rollapply(
          x, n, function(window) reference(window, w),
          align = align, fill = if (align == "center") NULL else NA
        ))
      )
    }
  }

  # the weight has to follow its own value through the sort
  expect_equal(roll_median(c(1, 2, 3), 3, weights = c(1, 1, 10)), 3)
  expect_equal(roll_median(c(3, 2, 1), 3, weights = c(10, 1, 1)), 3)

})

test_that("uniform weights take the unweighted path exactly", {

  # any uniform weight vector normalizes to exactly one in real arithmetic, so
  # these agree bitwise with the unweighted call -- including window sizes
  # whose rescaling rounds away from one in floating point (e.g. 1/49 * 49)
  set.seed(321)
  x <- rnorm(200)
  x[c(5, 20, 21, 100)] <- NA

  ops <- list(roll_sum, roll_mean, roll_min, roll_max, roll_prod, roll_var, roll_sd)
  for (op in ops) {
    for (n in c(3L, 6L, 49L, 64L)) {
      for (na.rm in c(TRUE, FALSE)) {
        expect_identical(op(x, n, weights = rep(1, n), na.rm = na.rm),
                         op(x, n, na.rm = na.rm))
        expect_identical(op(x, n, weights = rep(2.5, n), na.rm = na.rm),
                         op(x, n, na.rm = na.rm))
      }
    }
  }

  # without 'normalize', only a vector of ones is the unweighted call
  expect_identical(roll_sum(x, 8, weights = rep(1, 8), normalize = FALSE),
                   roll_sum(x, 8))
  expect_equal(roll_sum(x[101:108], 8, weights = rep(2, 8), normalize = FALSE),
               2 * sum(x[101:108]))

})

test_that("uniform weights route the weighted median to its lower form", {

  # a weighted median selects an observation: on an even window, the lower of
  # the two middle values, where the unweighted median averages them
  x <- c(1, 2, 3, 4, 5)
  expect_equal(roll_median(x, 4, weights = rep(1, 4)), c(2, 3))
  expect_equal(roll_median(x, 4), c(2.5, 3.5))

  lower_median <- function(window) {
    window <- window[!is.na(window)]
    if (!length(window)) return(NA_real_)
    sort(window)[(length(window) + 1) %/% 2]
  }

  # the routed path agrees with the reference on either side of the
  # incremental crossover, NAs and even windows included
  set.seed(987)
  y <- rnorm(300)
  y[sample(300, 30)] <- NA

  for (n in c(4L, 64L)) {
    windows <- seq_len(length(y) - n + 1L)
    expect_equal(
      roll_median(y, n, weights = rep(1, n), na.rm = TRUE),
      vapply(windows, function(i) lower_median(y[i:(i + n - 1L)]), numeric(1)))
  }

  # a 'by' past the crossover takes the from-scratch path through the same form
  n <- 8L
  starts <- seq(1L, length(y) - n + 1L, by = 7L)
  expect_equal(
    roll_median(y, n, weights = rep(1, n), by = 7, na.rm = TRUE),
    vapply(starts, function(i) lower_median(y[i:(i + n - 1L)]), numeric(1)))

})

test_that("the weighted median selection matches a sort-and-scan reference", {

  reference <- function(window, weights) {
    keep <- !is.na(window)
    window <- window[keep]
    weights <- weights[keep]
    if (!length(window)) return(NA_real_)
    ordered <- order(window)
    window <- window[ordered]
    weights <- weights[ordered]
    window[which(cumsum(weights) >= sum(weights) / 2)[1]]
  }

  rolled <- function(x, w) {
    n <- length(w)
    windows <- seq_len(length(x) - n + 1L)
    vapply(windows, function(i) reference(x[i:(i + n - 1L)], w), numeric(1))
  }

  # windows large enough to take the descent through several partitions;
  # continuous weights, so no crossing lands on a knife edge
  set.seed(654)
  x <- rnorm(400)
  x[sample(400, 25)] <- NA
  w <- runif(37)

  expect_equal(
    roll_median(x, 37, weights = w, na.rm = TRUE, normalize = FALSE),
    rolled(x, w))

  # repeated values exercise the pivot's whole-run partitioning
  y <- sample(round(rnorm(400), 1))
  expect_equal(
    roll_median(y, 37, weights = w, normalize = FALSE),
    rolled(y, w))

})

test_that("negative weights keep the sorted scan's crossing", {

  # a negative weight makes the cumulative weight non-monotonic, which the
  # partition-based selection cannot follow; those windows take the scan
  expect_equal(
    roll_median(c(5, 1, 3), 3, weights = c(1, -1, 2), normalize = FALSE), 3)
  expect_equal(
    roll_median(c(1, 2, 3), 3, weights = c(3, -2, 1), normalize = FALSE), 1)

  # an all-zero weight total still has no crossing to find
  expect_equal(
    roll_median(c(1, 2, 3), 3, weights = c(0, 0, 0), normalize = FALSE),
    NA_real_)

})

test_that("weighted roll_median respects na.rm", {
  x <- c(1, 2, NA, 4, 5, 6, 7)
  expect_equal(roll_median(x, 3, weights = c(1, 2, 1), na.rm = TRUE),
               c(2, 2, 4, 5, 6))
  expect_equal(roll_median(x, 3, weights = c(1, 2, 1), na.rm = FALSE),
               c(NA, NA, NA, 5, 6))
  # an all-NA window gives NA rather than reading past the window
  expect_equal(roll_median(rep(NA_real_, 4), 2, weights = c(1, 1), na.rm = TRUE),
               rep(NA_real_, 3))
})

context("accuracy")

# The rolling routines carry a window's state forward rather than reading the
# window afresh at every point. That is only sound while the arithmetic holds
# its digits, so these check the cases that make it hard: windows spanning
# enormous ranges of magnitude, and windows whose spread is tiny beside their
# mean.

hard_data <- function() {
  set.seed(11)
  list(
    # eighty orders of magnitude within a single window
    extreme = rnorm(200)^100,
    # occasional values that dwarf everything around them, then depart
    spikes  = { v <- rnorm(200); i <- seq(1, 200, by = 13); v[i] <- v[i] * 1e18; v },
    # deviations twelve orders of magnitude below the values themselves
    offset  = rnorm(200, mean = 1e10, sd = 1e-4),
    # totals that cancel to nothing. 2^45 rather than a round decimal so that
    # every partial sum of a window stays a whole number below 2^53, and so is
    # held exactly -- otherwise no summation order is the right one to expect
    cancel  = rep(c(2^45, -2^45, 1, -1), length.out = 200)
  )
}

test_that("rolling sums and means hold their precision on hard data", {

  for (x in hard_data()) {
    for (n in c(2L, 5L, 20L, 50L)) {
      windows <- seq_len(length(x) - n + 1L)
      # summing from the smallest magnitude upwards loses the least
      reference <- vapply(windows, function(i) {
        w <- x[i:(i + n - 1L)]
        sum(w[order(abs(w))])
      }, numeric(1))
      expect_equal(roll_sum(x, n), reference)
      expect_equal(roll_mean(x, n), reference / n)
    }
  }

})

test_that("rolling variances hold their precision on hard data", {

  for (x in hard_data()) {
    for (n in c(2L, 5L, 20L, 50L)) {
      windows <- seq_len(length(x) - n + 1L)
      # variance is shift invariant, so centring first is the same number
      # computed in a well conditioned way
      reference <- vapply(windows, function(i) {
        w <- x[i:(i + n - 1L)]
        var(w - w[1L])
      }, numeric(1))
      expect_equal(roll_var(x, n), reference)
      expect_equal(roll_sd(x, n), sqrt(reference))
    }
  }

})

test_that("a window whose deviations overflow gives an infinite variance", {

  # squaring these passes what a double can hold, as it does for var()
  # itself; the window sizes straddle the incremental crossover
  x <- rep(c(1e200, -2e200, 3e200, -1.5e200, 2.5e200), 5)
  for (n in c(3L, 16L)) {
    expect_equal(roll_var(x, n), rep(Inf, length(x) - n + 1))
    expect_equal(roll_sd(x, n), rep(Inf, length(x) - n + 1))
  }

})

test_that("infinities enter and leave a window without leaving a trace", {

  x <- c(1, Inf, 3, 4, 5, -Inf, 7, 8, 9)

  expect_equal(roll_sum(x, 2), c(Inf, Inf, 7, 9, -Inf, -Inf, 15, 17))
  expect_equal(roll_max(x, 2), c(Inf, Inf, 4, 5, 5, 7, 8, 9))
  expect_equal(roll_min(x, 2), c(1, 3, 3, 4, -Inf, -Inf, 7, 8))

  # a window holding both is a NaN, and the windows after it are not
  expect_true(is.nan(roll_sum(c(Inf, -Inf, 1, 2), 2)[1]))
  expect_equal(roll_sum(c(Inf, -Inf, 1, 2), 2)[3], 3)

})

test_that("infinities enter and leave large windows too", {

  # the small-window cases above sit below the incremental crossovers; these
  # sizes take the accumulators through their infinity bookkeeping as well
  set.seed(7)
  x <- rnorm(400)
  x[seq(10, 390, by = 37)] <- Inf
  x[seq(25, 390, by = 53)] <- -Inf

  for (n in c(13L, 128L)) {
    windows <- seq_len(length(x) - n + 1L)
    expect_equal(roll_sum(x, n),
                 vapply(windows, function(i) sum(x[i:(i + n - 1L)]), numeric(1)))
    expect_equal(roll_min(x, n),
                 vapply(windows, function(i) min(x[i:(i + n - 1L)]), numeric(1)))
    expect_equal(roll_max(x, n),
                 vapply(windows, function(i) max(x[i:(i + n - 1L)]), numeric(1)))
    expect_equal(roll_var(x, n),
                 vapply(windows, function(i) var(x[i:(i + n - 1L)]), numeric(1)))
  }

})

test_that("a total that overflows recovers once the culprits leave", {

  # Every value is finite, but early window totals overflow; subtracting a
  # finite value from that infinity leaves it infinite, so the windows past
  # the overflow have to be rebuilt rather than slid into. The window sizes
  # sit above the incremental crossovers, which is the only place a total is
  # carried forward at all.
  x <- c(rep(1e307, 80), rep(1, 200))
  expect_equal(tail(roll_sum(x, 64), 3), rep(64, 3))
  expect_equal(tail(roll_mean(x, 64), 3), rep(1, 3))

  # likewise for the variance: these squared deviations are individually
  # finite, but their running sum is not
  x <- c(0, rep(1.2e154, 30), rep(1, 60))
  expect_equal(tail(roll_var(x, 16), 3), rep(0, 3))
  expect_equal(tail(roll_sd(x, 16), 3), rep(0, 3))

})

test_that("a NaN keeps its identity through a sum, an NA likewise", {

  # the incremental path counts its missing values rather than adding them
  # up, so the counting has to preserve the distinction the arithmetic did;
  # the window sizes straddle the incremental crossover
  for (n in c(10L, 112L)) {
    x <- c(NaN, rnorm(150))
    expect_true(is.nan(roll_sum(x, n)[1]))
    expect_true(is.nan(roll_mean(x, n)[1]))

    x <- c(NA, rnorm(150))
    expect_true(is.na(roll_sum(x, n)[1]) && !is.nan(roll_sum(x, n)[1]))
    expect_true(is.na(roll_mean(x, n)[1]) && !is.nan(roll_mean(x, n)[1]))
  }

})

test_that("the sign of a zero survives min, max and median", {

  # compared through their reciprocals, since identical() treats 0 and -0 as
  # equal but 1/0 and 1/-0 are distinguishable infinities
  x <- c(0, -0, 1, -0, 0)

  # min keeps the earlier of two equal values, which matches base R
  expect_identical(
    1 / roll_min(x, 2),
    1 / vapply(1:4, function(i) min(x[i:(i+1L)]), numeric(1)))

  # max has always kept the later, so a zero maximum takes the later zero's
  # sign -- unlike base R's max, which keeps the first
  expect_identical(1 / roll_max(x, 2), c(-Inf, 1, 1, Inf))

  # Sliding removes zeros by value, so the removal has to match the sign bit
  # or the sorted window drifts away from the data. Only even windows are
  # asserted: their median averages both middle values, where an odd window's
  # choice among equal-comparing zeros is unspecified even in base R.
  x <- c(-0.0, 0.0, -0.0, 0.0, -0.0)
  expect_identical(
    1 / roll_median(x, 2),
    1 / vapply(1:4, function(i) median(x[i:(i+1L)]), numeric(1)))

})

context("n and weights")

test_that("supplying both 'n' and 'weights' with differing lengths warns (#39)", {

  # 'weights' wins, so a mismatched 'n' is silently discarded without this
  expect_warning(roll_sum(1:9, n = 2, weights = 1), "'n' is ignored")
  expect_warning(roll_sum(1:9, 2, 1), "'n' is ignored")
  expect_warning(roll_sumr(1:5, n = 3, weights = c(1, 1)), "'n' is ignored")
  expect_warning(roll_meanl(1:5, n = 3, weights = c(1, 1)), "'n' is ignored")
  expect_warning(roll_var(1:6, n = 9, weights = c(1, 1, 1)), "'n' is ignored")

  # the message reports both the window used and the one asked for
  expect_warning(roll_sum(1:9, n = 2, weights = 1),
                 "using 'n = 1' rather than 'n = 2'")

})

test_that("no warning when 'n' and 'weights' agree, or only one is given", {

  expect_silent(roll_sum(1:9, n = 2, weights = c(1, 1)))
  expect_silent(roll_sum(1:9, weights = c(1, 1)))
  expect_silent(roll_sum(1:9, n = 2))
  expect_silent(roll_sum(1:9))
  expect_silent(roll_meanr(1:9, n = 3, weights = c(1, 2, 3)))

})

test_that("the warning does not change the computed result (#39)", {

  expect_equal(
    suppressWarnings(roll_sum(1:9, n = 2, weights = 1)),
    roll_sum(1:9, n = 1, weights = 1)
  )

  # still the length-of-weights window, as before
  expect_equal(
    suppressWarnings(roll_sumr(1:5, n = 3, weights = c(1, 1))),
    roll_sumr(1:5, n = 2, weights = c(1, 1))
  )

})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

na_locf <- function(x) {
    .Call(`_RcppRoll_na_locf`, x)
}

roll_mean_impl <- function(x, n, weights, by, fill_, partial, align, normalize, na_rm) {
    .Call(`_RcppRoll_roll_mean_impl`, x, n, weights, by, fill_, partial, align, normalize, na_rm)
}

roll_median_impl <- function(x, n, weights, by, fill_, partial, align, normalize, na_rm) {
    .Call(`_RcppRoll_roll_median_impl`, x, n, weights, by, fill_, partial, align, normalize, na_rm)
}

roll_min_impl <- function(x, n, weights, by, fill_, partial, align, normalize, na_rm) {
    .Call(`_RcppRoll_roll_min_impl`, x, n, weights, by, fill_, partial, align, normalize, na_rm)
}

roll_max_impl <- function(x, n, weights, by, fill_, partial, align, normalize, na_rm) {
    .Call(`_RcppRoll_roll_max_impl`, x, n, weights, by, fill_, partial, align, normalize, na_rm)
}

roll_prod_impl <- function(x, n, weights, by, fill_, partial, align, normalize, na_rm) {
    .Call(`_RcppRoll_roll_prod_impl`, x, n, weights, by, fill_, partial, align, normalize, na_rm)
}

roll_sum_impl <- function(x, n, weights, by, fill_, partial, align, normalize, na_rm) {
    .Call(`_RcppRoll_roll_sum_impl`, x, n, weights, by, fill_, partial, align, normalize, na_rm)
}

roll_sd_impl <- function(x, n, weights, by, fill_, partial, align, normalize, na_rm) {
    .Call(`_RcppRoll_roll_sd_impl`, x, n, weights, by, fill_, partial, align, normalize, na_rm)
}

roll_var_impl <- function(x, n, weights, by, fill_, partial, align, normalize, na_rm) {
    .Call(`_RcppRoll_roll_var_impl`, x, n, weights, by, fill_, partial, align, normalize, na_rm)
}

# Report the OpenMP state once, on attach. Suppressible in the usual way
# through suppressPackageStartupMessages(), or universally with
# options(RcppRoll.quiet = TRUE).
.onAttach <- function(libname, pkgname) {

  if (isTRUE(getOption("RcppRoll.quiet")))
    return(invisible())

  threads <- roll_threads()
  if (is.na(threads)) {
    packageStartupMessage(
      "RcppRoll was compiled without OpenMP support; rolling computations ",
      "will run on a single thread. See the README for enabling OpenMP."
    )
  } else {
    packageStartupMessage(sprintf(
      "RcppRoll: using up to %d thread%s; control with options(RcppRoll.threads = <n>)",
      threads, if (threads == 1L) "" else "s"
    ))
  }

}

#' RcppRoll
#'
#' This package implements a number of 'roll'-ing functions for \R
#' vectors and matrices.
#'
#' Currently, the exported functions are:
#'  \itemize{
#'  \item{\code{\link{roll_max}}}
#'  \item{\code{\link{roll_mean}}}
#'  \item{\code{\link{roll_median}}}
#'  \item{\code{\link{roll_min}}}
#'  \item{\code{\link{roll_prod}}}
#'  \item{\code{\link{roll_sd}}}
#'  \item{\code{\link{roll_sum}}}
#'  \item{\code{\link{roll_var}}}
#'  }
#'
#' @section Parallelization:
#'
#' When the package is compiled with \code{OpenMP} support, the rolling
#' window computations are parallelized across threads. By default, the
#' number of threads is chosen by the \code{OpenMP} runtime (typically
#' controlled through the \code{OMP_NUM_THREADS} environment variable);
#' it can be set explicitly with \code{options(RcppRoll.threads = <n>)},
#' and parallelization can be disabled with
#' \code{options(RcppRoll.threads = 1)}. Small inputs are always computed
#' serially, and results are identical whatever the number of threads --
#' including on builds without \code{OpenMP} support at all.
#'
#' Use \code{\link{roll_threads}()} to check how many threads are in use,
#' or whether the installed package has \code{OpenMP} support at all. The
#' package also reports this when attached; suppress that message with
#' \code{options(RcppRoll.quiet = TRUE)}.
#'
#' @useDynLib RcppRoll, .registration = TRUE, .fixes = "C_"
"_PACKAGE"

#' Report the Number of Threads Used for Rolling Computations
#'
#' Reports how many threads the rolling computations would put to work on a
#' large enough input. The count honors \code{options(RcppRoll.threads)}
#' where that is set, and otherwise defers to the \code{OpenMP} runtime
#' default (typically controlled through the \code{OMP_NUM_THREADS}
#' environment variable). Small inputs are always computed serially, and
#' results do not depend on the number of threads used.
#'
#' @return An integer scalar: the maximum number of threads used, or
#'   \code{NA} if the package was compiled without \code{OpenMP} support --
#'   useful for checking what an installation from sources ended up with.
#'
#' @export
roll_threads <- function() {
  .Call(C_roll_threads_impl)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/threads.R
\name{roll_threads}
\alias{roll_threads}
\title{Report the Number of Threads Used for Rolling Computations}
\usage{
roll_threads()
}
\value{
An integer scalar: the maximum number of threads used, or
  \code{NA} if the package was compiled without \code{OpenMP} support --
  useful for checking what an installation from sources ended up with.
}
\description{
Reports how many threads the rolling computations would put to work on a
large enough input. The count honors \code{options(RcppRoll.threads)}
where that is set, and otherwise defers to the \code{OpenMP} runtime
default (typically controlled through the \code{OMP_NUM_THREADS}
environment variable). Small inputs are always computed serially, and
results do not depend on the number of threads used.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppRoll-package.R
\docType{package}
\name{RcppRoll}
\alias{RcppRoll}
\alias{RcppRoll-package}
\title{RcppRoll}
\description{
This package implements a number of 'roll'-ing functions for \R
vectors and matrices.
}
\details{
Currently, the exported functions are:
 \itemize{
 \item{\code{\link{roll_max}}}
 \item{\code{\link{roll_mean}}}
 \item{\code{\link{roll_median}}}
 \item{\code{\link{roll_min}}}
 \item{\code{\link{roll_prod}}}
 \item{\code{\link{roll_sd}}}
 \item{\code{\link{roll_sum}}}
 \item{\code{\link{roll_var}}}
 }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppRoll-package.R
\docType{package}
\name{RcppRoll-package}
\alias{RcppRoll}
\alias{RcppRoll-package}
\title{RcppRoll}
\description{
This package implements a number of 'roll'-ing functions for \R
vectors and matrices.
}
\details{
Currently, the exported functions are:
 \itemize{
 \item{\code{\link{roll_max}}}
 \item{\code{\link{roll_mean}}}
 \item{\code{\link{roll_median}}}
 \item{\code{\link{roll_min}}}
 \item{\code{\link{roll_prod}}}
 \item{\code{\link{roll_sd}}}
 \item{\code{\link{roll_sum}}}
 \item{\code{\link{roll_var}}}
 }
}
\section{Parallelization}{


When the package is compiled with \code{OpenMP} support, the rolling
window computations are parallelized across threads. By default, the
number of threads is chosen by the \code{OpenMP} runtime (typically
controlled through the \code{OMP_NUM_THREADS} environment variable);
it can be set explicitly with \code{options(RcppRoll.threads = <n>)},
and parallelization can be disabled with
\code{options(RcppRoll.threads = 1)}. Small inputs are always computed
serially, and results are identical whatever the number of threads --
including on builds without \code{OpenMP} support at all.

Use \code{\link{roll_threads}()} to check how many threads are in use,
or whether the installed package has \code{OpenMP} support at all. The
package also reports this when attached; suppress that message with
\code{options(RcppRoll.quiet = TRUE)}.
}

\seealso{
Useful links:
\itemize{
  \item \url{https://kevinushey.github.io/RcppRoll/}
  \item \url{https://github.com/kevinushey/RcppRoll}
  \item Report bugs at \url{https://github.com/kevinushey/RcppRoll/issues}
}

}
\author{
\strong{Maintainer}: Kevin Ushey \email{kevinushey@gmail.com}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppRoll.R
\name{RcppRoll-exports}
\alias{RcppRoll-exports}
\alias{roll_mean}
\alias{roll_meanr}
\alias{roll_meanl}
\alias{roll_median}
\alias{roll_medianr}
\alias{roll_medianl}
\alias{roll_min}
\alias{roll_minr}
\alias{roll_minl}
\alias{roll_max}
\alias{roll_maxr}
\alias{roll_maxl}
\alias{roll_prod}
\alias{roll_prodr}
\alias{roll_prodl}
\alias{roll_sum}
\alias{roll_sumr}
\alias{roll_suml}
\alias{roll_sd}
\alias{roll_sdr}
\alias{roll_sdl}
\alias{roll_var}
\alias{roll_varr}
\alias{roll_varl}
\title{RcppRoll}
\usage{
roll_mean(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = numeric(0),
  partial = FALSE,
  align = c("center", "left", "right"),
  normalize = TRUE,
  na.rm = FALSE
)

roll_meanr(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = NA,
  partial = FALSE,
  align = "right",
  normalize = TRUE,
  na.rm = FALSE
)

roll_meanl(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = NA,
  partial = FALSE,
  align = "left",
  normalize = TRUE,
  na.rm = FALSE
)

roll_median(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = numeric(0),
  partial = FALSE,
  align = c("center", "left", "right"),
  normalize = TRUE,
  na.rm = FALSE
)

roll_medianr(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = NA,
  partial = FALSE,
  align = "right",
  normalize = TRUE,
  na.rm = FALSE
)

roll_medianl(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = NA,
  partial = FALSE,
  align = "left",
  normalize = TRUE,
  na.rm = FALSE
)

roll_min(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = numeric(0),
  partial = FALSE,
  align = c("center", "left", "right"),
  normalize = TRUE,
  na.rm = FALSE
)

roll_minr(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = NA,
  partial = FALSE,
  align = "right",
  normalize = TRUE,
  na.rm = FALSE
)

roll_minl(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = NA,
  partial = FALSE,
  align = "left",
  normalize = TRUE,
  na.rm = FALSE
)

roll_max(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = numeric(0),
  partial = FALSE,
  align = c("center", "left", "right"),
  normalize = TRUE,
  na.rm = FALSE
)

roll_maxr(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = NA,
  partial = FALSE,
  align = "right",
  normalize = TRUE,
  na.rm = FALSE
)

roll_maxl(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = NA,
  partial = FALSE,
  align = "left",
  normalize = TRUE,
  na.rm = FALSE
)

roll_prod(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = numeric(0),
  partial = FALSE,
  align = c("center", "left", "right"),
  normalize = TRUE,
  na.rm = FALSE
)

roll_prodr(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = NA,
  partial = FALSE,
  align = "right",
  normalize = TRUE,
  na.rm = FALSE
)

roll_prodl(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = NA,
  partial = FALSE,
  align = "left",
  normalize = TRUE,
  na.rm = FALSE
)

roll_sum(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = numeric(0),
  partial = FALSE,
  align = c("center", "left", "right"),
  normalize = TRUE,
  na.rm = FALSE
)

roll_sumr(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = NA,
  partial = FALSE,
  align = "right",
  normalize = TRUE,
  na.rm = FALSE
)

roll_suml(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = NA,
  partial = FALSE,
  align = "left",
  normalize = TRUE,
  na.rm = FALSE
)

roll_sd(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = numeric(0),
  partial = FALSE,
  align = c("center", "left", "right"),
  normalize = TRUE,
  na.rm = FALSE
)

roll_sdr(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = NA,
  partial = FALSE,
  align = "right",
  normalize = TRUE,
  na.rm = FALSE
)

roll_sdl(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = NA,
  partial = FALSE,
  align = "left",
  normalize = TRUE,
  na.rm = FALSE
)

roll_var(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = numeric(0),
  partial = FALSE,
  align = c("center", "left", "right"),
  normalize = TRUE,
  na.rm = FALSE
)

roll_varr(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = NA,
  partial = FALSE,
  align = "right",
  normalize = TRUE,
  na.rm = FALSE
)

roll_varl(
  x,
  n = 1L,
  weights = NULL,
  by = 1L,
  fill = NA,
  partial = FALSE,
  align = "left",
  normalize = TRUE,
  na.rm = FALSE
)
}
\arguments{
\item{x}{A numeric vector or a numeric matrix.}

\item{n}{The window size. Ignored when \code{weights} is non-\code{NULL}.}

\item{weights}{A vector of length \code{n}, giving the weights for each
element within a window. If \code{NULL}, we take unit weights of width \code{n}.}

\item{by}{Calculate at every \code{by}-th point rather than every point.}

\item{fill}{Either an empty vector (no fill), or a vector (recycled to)
length 3 giving left, center and right fills.}

\item{partial}{Compute windows at the edges of \code{x} over however
many elements are in range, rather than filling them? Cannot be
combined with \code{weights}, and \code{fill} does not apply.}

\item{align}{Align windows on the \code{"left"}, \code{"center"} or
\code{"right"}.}

\item{normalize}{Normalize window weights, such that they sum to \code{n}.}

\item{na.rm}{Remove missing values?}
}
\description{
Efficient windowed / rolling operations. Each function
here applies an operation over a moving window of
size \code{n}, with (customizable) weights specified
through \code{weights}.
}
\details{
The functions postfixed with \code{l} and \code{r}
are convenience wrappers that set \strong{l}eft
/ \strong{r}ight alignment of the windowed operations.
}
